/*
 * Actions for basic rule operations
 */
var sensorData = require('../../libs/sensorData.js')
  , sensorHelper = require('../helpers/sensorHelper')
  , actionData = require('../../libs/actionData.js')
  , _ = require('underscore')
  , mongoose = require('mongoose')
  , Rule = mongoose.model('Rule');


/*
 * Index action
 */
exports.index = function(req, res) {
  res.render('site/rule/basic', {
    rules: req.jsonResponse.rules,
    sensors: req.jsonResponse.sensors,
    hooks: req.jsonResponse.hooks,
    sensorData: sensorData,
    sensorHelper: sensorHelper});
}


/*
 * Create action
 */
exports.create = function(req, res, next) {
  var rule;
  try {
    rule = createRuleFromRequest(req);
  } catch (e) {
    req.jsonResponse.success = false;
    req.jsonResponse.message = e.message;
    next();
    return;
  }
  
  rule.save(function(err, doc) {
    if (err) {
      req.jsonResponse.success = false;
      req.jsonResponse.message = e.message;
    } else {
      req.jsonResponse.success = true;
    }

    next();
  });
}

/* Create a rule object from a request */
function createRuleFromRequest(req) {
  var rawIfs;

  // base properties
  var rule = {
    name: req.body.name,
    type: req.body.type,
    user: req.user._id
  };
  
  // properties that depend on type
  if (rule.type == 'sensor') {
    rule.sensorId = req.body.sensor['type-'+req.body.sensorType].sensorId;
    rawIfs = req.body.sensor['type-'+req.body.sensorType].ifs;
  } else if (rule.type == 'hook') {
    rule.hookId = req.body.hookId;
    rawIfs = {};
  } else {
    throw new Error("Unsupported rule type");
  }
  
  // ifs
  var ifs = [];
  var opers = ['<', '>=', '<=', '==', '>', '!=', '!==', '==='];
  for (var l in rawIfs) {
    var r = rawIfs[l].value;
    var oper = rawIfs[l].operator;
    if (r !== null && r !== undefined && r !== '' && opers.indexOf(oper) !== -1)
      ifs.push({operands: [('_object.'+l), {'_value': convertTestValue(r)}], test: oper});
  }
  
  if (ifs.length == 0)
    ifs.push({operands: [{'_value': 1},{'_value': 1}], test: '=='});
  
  // then
  var actionName = req.body.actionType;
  var rawArgs = req.body.action[actionName];
  var action = _.find(actionData.actions, function(action) { return action.type == actionName; });
  
  if (!action)
    throw new Error("Unknown action type \""+actionName+"\"");
  
  var args = [];
  
  action.args.forEach(function (arg,argPos) {
    if (arg.type == 'uid') {
      args.push({"_value": req.user._id});
    } else if (arg.type == 'literal') {
      args.push(arg.value);
    } else {
      args.push({"_value": rawArgs[argPos]});
    }
  });
  
  var then = {_catalog: {name: actionName, args: args}};
  
  rule.rule = {"ifs": ifs, "then": then};
  
  return new Rule(rule);
}

/* for now, a best guess at data type conversion for rule if values */
function convertTestValue(value) {
  if (value == 'true') 
    return true;
  else if (value == 'false')
    return false;
  else if (/^-?\d+$/.test(value))
    return parseInt(value);
  else
    return value;
}
