/**
 * Module dependencies.
 */
var passport = require('passport')
, mongoose = require('mongoose')
, crypto = require('crypto')
, Gateway = mongoose.model('Gateway')
, config = require('../../config/conftool.js').getConf();

exports.findOrCreate = function(req, res, next){
  // req.user should already be filled with the users information
  
  var findGateway = function(id){
    console.log("Finding gateway", id, req.user._id);
    Gateway.findOne({_id: id, user: req.user._id})
    .exec(function(err, gw){
      if(err) return next(err);
      if(!gw)
        return createGateway();
      console.log("found gw", gw);
      req.gw = gw;
      next();
    });
  };
  
  var createGateway=  function(){
    console.log('Creating gateway');
    var gw = new Gateway({user: req.user._id, status: {state: "offline"}});
    gw.save(function(err, doc){
      console.log("Created gateway", gw);
      if(err) return next(err);
      req.gw = doc;
      next();
    }); 
  };

  console.log(req.body);
  if(req.body.gatewayId){
    findGateway(req.body.gatewayId);
  }else{
    createGateway();
  }
};
exports.updateName = function(req, res, next){
  var gateway = req.gateway;
  if(gateway && req.body.name){
    gateway.name = req.body.name;
    gateway.save(function(err, doc){
      if(err) return next(err);
      var response = {result: 'success', redraw :true};
      req.jsonResponse = response;
      next();
    });
  }
  else{
    var response = {result: 'failure',redraw : false};
    req.jsonResponse = response;
    next();
  }
};
exports.update = function(event, next){
  console.log("Internal event service request to update gateway: ", event);
    Gateway.findOne({_id: event.gatewayId})
    .exec(function(err, gw){
      if(err) return next(err);
      if(!gw){
        console.log("Internal event service request failed: ", event);
        return next();
      }
      gw.drivers = event.payload.drivers;
      gw.status = event.payload.status;
      gw.save(function(err, doc){
        console.log("Updated gateway", doc);
        if(err) return next(err);
        return next();
      }); 
    });
}

exports.socketkey = function(req, res){
  
    var socketKey = {
                      user :req.user,
                      timestamp: new Date()
                    };
    var gwId = null;
    if(req.gw && req.gw._id) {
      socketKey.gatewayId = req.gw._id;
      gwId = req.gw._id;
    }


    var cipher = crypto.createCipher("aes-256-cbc",config.socketPass);
    var encryptedSocketKey =  cipher.update(JSON.stringify(socketKey), 'utf8', 'base64');
    encryptedSocketKey +=  cipher.final('base64');
    res.json({status: 'ok', gatewayId: gwId, 
             timestamp: socketKey.timestamp, socketKey: encryptedSocketKey});

};

exports.decryptSocketKey = function(encryptedSocketKey, cb){
  var decipher = crypto.createDecipher('aes-256-cbc', "SecretPassword");
  var user = decipher.update(encryptedSocketKey, 'base64', 'utf8');
  user += decipher.final('utf8');
  cb(JSON.parse(user));
};

