
var passport = require('passport')
, mongoose = require('mongoose')
, actions = require('../../libs/sensorData.js')
, User = mongoose.model('User')
, Sensor = mongoose.model('Sensor')
, Reading = mongoose.model('Reading')
, config = require('../../config/config')['development']
, GrEvents = require('gr-events')
, event_service = new GrEvents({http: null, listeners: config.grevents});

exports.get = function(req, res, next){
  if(req.sensor){
    req.jsonResponse = req.sensor;
    next();
  }else{
    next(new Error("No sensor found"));
  }
}
exports.getActions = function(req, res, next){
  var type = req.sensor.lastReading.type;
  if(type && actions[type]){
    req.jsonResponse  = {actions: actions[type].actions, sensor: req.sensor};
    next();
  }else{
    req.jsonResponse = {sensor: req.sensor, actions: []};
    next();
  }
}
exports.update = [
  function(req, res, next){
  var sensor = req.sensor;
    if(sensor && req.body.name){
      sensor.name = req.body.name;
      sensor.save(function(err, doc){
        if(err) throw (err);
        var response = {result: 'success',redraw : true}
        req.jsonResponse = response;
        next(); 
      });
    }else{
        var type = req.sensor.lastReading.type;
        res.render('sensors/' + type + '/show', {sensor: req.sensor});
    }
  }
]
exports.doAction = function(req, res, next){
  //normallize against different versions so body always contains all values
  req.body.name = req.params.action; 
  req.body.mac = req.sensor.mac;
  req.body.user = req.user._id;
  if(actions[req.sensor.lastReading.type]){
    var action = {type: 'action', user: req.sensor.user,
                  payload: { mac: req.sensor.mac, name: req.params.action, values : req.body }};
    event_service.sendToListeners(action, function(err, res){
      console.log("Sent action request out to all listeners", res, err); 
    });

  
    console.log(action);
    var response = {result: 'success',redraw : true}
    req.jsonResponse = response;
    return next();
  }else{
    console.log('action not supported');
    return next(new Error('action not supported'));
  }
}
exports.showactions = [
  function(req, res){
  var type = req.sensor.lastReading.type;
    if(type && actions[type]){
      console.log('creating action forms for sensor type ', actions[type].sensorType);
      res.render('sensors/action', 
                 {actions: actions[type].actions, 
                  type: actions[type].sensorType,
                  sensor: req.sensor
                 }
                );
    }else{
      console.log('sensor type not supported yet ', type);
      res.render('sensors/' + type + '/show', {sensor: req.sensor});
    }

  }
]
exports.show = [
  function(req, res){
  if(req.sensor && req.sensor.lastReading.type){
    var type = req.sensor.lastReading.type;
    console.log(req.sensor);
    res.render('sensors/' + type + '/show', {sensor: req.sensor});
  }else{
    console.log('sensor not found or no type', req.sensor, req.sensor.lastReading.type);
    res.redirect('/notfound');
  }
}
]
exports.remove = function(req, res, next){
  var sensor = req.sensor;
  sensor.remove(function(err){
    if(err) return next(err);
    else
      req.jsonResponse.succes = true;
    next();
  });
}
//Controller for sensor API
exports.read = [
  function(req, res) {
    if(req.sensor){
      console.log('sensor already exists, reading');
      var sensor = req.sensor;
      sensor.lastReading = req.body.payload;

      var event = {type: 'reading', payload: sensor, user: req.user._id};
      event_service.sendToListeners(event, function(err, res){
        console.log("sent requests to listeners", err); 
      
      });
      sensor.save(function(err){
        if(err)
          console.log(err);
      });
      var reading = new Reading({ data: req.body.payload,
                                  sensor: sensor._id});
      reading.save(function(err){
        if(err)
          console.log(err);
      });
    }else{
      if(!req.body.sensorId){
        return res.send(400, "Missing sensorId");
      }
      var sensor = new Sensor({mac: req.body.sensorId,
                               name: 'New Sensor',
                               user: req.user._id,
                               lastReading: req.body.payload});
      

      var event = {type: 'reading', payload: sensor, user: req.user._id};
      
      event_service.sendToListeners(event, function(err, res){});
      sensor.save(function(err){
        if(err) {
          console.log(err);
        }
        console.log(this);
      });
    }
    res.json({ status: 'OK'});
  }
]
