/**
 * Module dependencies.
 */
var passport = require('passport')
, mongoose = require('mongoose')
, crypto = require('crypto')
, User = mongoose.model('User')
, Sensor = mongoose.model('Sensor')
, Location = mongoose.model('Location')
, Gateway = mongoose.model('Gateway')
, Rule = mongoose.model('Rule')
, config = require('../../config/conftool.js').getConf()
, Hook = mongoose.model('Hook');

exports.signup = function (req, res){
  res.render('site/user/signup', {
              title: 'Sign up'
              });
}
exports.create = function(req, res){
 console.log(req.body); 
  var user = new User(req.body);
  console.log(user);
  user.provider = 'local';
  user.save(function(err){
    if(err) {
      console.log(err);
      return res.render('site/user/signup', {
        error: err.errors
      });
    }
    console.log(this);
    req.logIn(user, function(err){
      if(err) return next(err);
      return res.redirect('/');
    });
  });
}
exports.addHook = function(req, res, next){
  var user = req.user;
  var hook = req.account;
  if(req.account){
    user.hooks[hook.provider] = hook;
    user.save(function(err){
      if(err) return next(err);
      else if(!user) next(new Error("no user returned. user should be logged in"));
      else{
        Hook.findOne({lookupId: hook.provider + '-' + hook.username}, 
          function(err, obj){
            if(err) return next(err);
            else{
              if(obj){
                obj.data = hook._json;
                obj.key = hook.key;
                obj.save(function(err){
                  return next();
                });
              }else{
                var name = '';
                switch(hook.provider){
                  case 'github':
                    name = 'Github push';
                  break;
                  case 'webrtcgw':
                    name = 'Incoming Call';
                  break;
                  default:
                    name = 'hook';
                  break;
                }
                var obj  = new Hook({provider: hook.provider,
                                     providerId: hook.username,
                                     user: user._id,
                                     data: hook._json,
                                     key: hook.key,
                                     name: name
                                    });
                obj.save(function(err){
                  return next();
                });
              }
            }
          });
      }
    });
  }else{
    return next(new Error("no hook in request, failing"));
  }
}




exports.session = function(req, res, next){
  passport.authenticate('local', function(err, user, info){
    if(err) { return next(err)}
    if(!user) {
      if(req.isMobile || req.isTablet)
        return res.redirect('/mobile');
      else
        return res.redirect('/login');
    }else{
      req.logIn(user, function(err){
        if(err) {return next(err);}
        if(req.session.returnTo){
          console.log('using returnTo');
          return res.redirect(req.session.returnTo);  
        } 
        else if(req.isMobile || req.isTablet){
            return res.redirect('/mobile/sensors');
        }
       else{
          return res.redirect('/');
        }
      });
    }})(req, res, next);
}
exports.about = function(req, res){
  console.log(req.session); 
  res.render('about/show');
}
exports.login = function(req, res) {
    res.render('site/user/login', {
          title: 'Login'
            });
}

exports.logout = function(req, res) {
    req.logout();
      res.redirect('/login');
}

exports.show = function(req, res) {
    var user = req.profile
      res.render('site/user/show', {
            title: user.name,
            user: user,
            updated: false
      });
}
//deprecated
exports.showallsensors = function(req, res){
  Sensor.find({user: req.user._id}, function(err, sensors){
    if(err) return err;
    res.render('site/user/allsensors', {sensors: sensors});
  });
}
exports.getallgateways = function(req, res, next){
  Gateway.find({user: req.user._id}, function(err, gateways){
    console.log("Got gateways", gateways);
    if(err) return err;
    if(gateways){
      req.jsonResponse.gateways = gateways;
    }else{
      req.jsonResponse.gateways = []; 
    }
    return next();
  });
}
exports.getalllocations = function(req, res, next){
  Location.find({user: req.user._id}, function(err, locations){
    console.log("Got locations", locations);
    if(err) return err;
    if(locations){
      req.jsonResponse.locations = locations;
    }else{
      req.jsonResponse.locations = []; 
    }
    return next();
  });
}

exports.getallsensors = function(req, res, next){
  Sensor.find({user: req.user._id}, function(err, sensors){
    if(err) return err;
    if(sensors){
      req.jsonResponse.sensors = sensors;
    }else{
      req.jsonResponse.sensors = []; 
    }
    return next();
  });
}
exports.getallhooks = function(req, res, next){
  console.log('looking up hook with "' + req.user._id + '"');
  Hook.find({user: req.user._id}, function(err, hooks){
    console.log('found some hooks', hooks);
    if(err) return err;
    if(hooks)
      req.jsonResponse.hooks = hooks;
    else
      req.jsonResponse.hooks = [];
    return next();
  });
}
exports.getallrules = function(req, res, next){
  Rule.find({user: req.user._id}, function(err, rules){
    if(err) return err;
    if(rules)
      req.jsonResponse.rules = rules;
    else
      req.jsonReponse.rules = [];
    return next();
  });
}
exports.signin = function(req, res) {}

exports.info = [
  function(req, res) {
    console.log(req.user);
    res.json({ user_id: req.user.id, name: req.user.name, phoneNumber: req.user.phoneNumber })
  }
]
exports.socketkey = [
  function(req, res){
    var socketKey = {
                      user :req.user,
                      timestamp: new Date()
                    };
    var cipher = crypto.createCipher("aes-256-cbc",config.socketPass);
    var encryptedSocketKey =  cipher.update(JSON.stringify(socketKey), 'utf8', 'base64');
    encryptedSocketKey +=  cipher.final('base64');
    res.json({status: 'ok', timestamp: socketKey.timestamp, socketKey: encryptedSocketKey});

}
]
exports.decryptSocketKey = function(encryptedSocketKey, cb){
  var decipher = crypto.createDecipher('aes-256-cbc', "SecretPassword");
  var user = decipher.update(encryptedSocketKey, 'base64', 'utf8');
  user += decipher.final('utf8');
  cb(JSON.parse(user));
}

