// user schema

var mongoose = require('mongoose')
  , Schema = mongoose.Schema
  , _ = require('underscore')

var types = 'time sensor hook'.split(' ');

//Rules can be associated with a sensor event, hook event, or time event
//type defines which event they are triggered by.
//lookup is either by sensor, hook, time, or user. The indexing causes WRITE performance degradation
//The index will greatly increase READ performance which is the goal.


var RuleSchema = new Schema({
    name: String
  , sensorId : {type : Schema.ObjectId, ref: 'Sensor'}
  , user: {type : Schema.ObjectId, ref: 'User'}
  , hookId : {type : Schema.ObjectId, ref: 'Hook'}
  , rule: {type: Schema.Types.Mixed, default: {}}
  , time: {type: Number }
  , createdAt : {type : Date, default : Date.now}
  , type: {type: String, enum: types}
  , enabled: {type: Boolean, default: true}
});

RuleSchema.index({hookId: 1, time: 1, sensorId: 1, user: 1});

mongoose.model('Rule', RuleSchema);
