/**
 * Module dependencies.
 */
var mongoose = require('mongoose'),
    User = mongoose.model('User'),
    Sensor = mongoose.model('Sensor'),
    Gateway = mongoose.model('Gateway'),
    Location = mongoose.model('Location'),
    Rule = mongoose.model('Rule'),
    login = require('connect-ensure-login');
  
module.exports = function(app, passport, event_service){



var site = require('../app/controllers/site');
var user = require('../app/controllers/user.js');
var admin = require('../app/controllers/admin.js');
var oauth = require('../app/controllers/oauth.js');
var sensor = require('../app/controllers/sensor.js');
var gateway = require('../app/controllers/gateway.js');
var mobile = require('../app/controllers/mobile.js');
var rule = require('../app/controllers/rule.js');
var location = require('../app/controllers/location.js');
var basicRule = require('../app/controllers/basicrule.js');
var multiParty = require("connect-multiparty");

var mpMiddleware = multiParty({uploadDir: "/mnt/tmp", autoFiles: true});

app.get('/',  user.about);
app.get('/login', site.loginForm);
app.post('/login', user.login);
app.get('/logout', site.logout);
app.get('/account', site.account);
app.get('/signup', user.signup);
app.post('/users', user.create);
app.get('/users/:userId', user.show);
app.post('/users/session', user.session);
app.get('/dialog/authorize', oauth.userAuthorization);
app.post('/dialog/authorize/decision', oauth.userDecision);
app.post('/oauth/request_token', oauth.requestToken);
app.post('/oauth/access_token', oauth.accessToken);

app.all('/auth/*', login.ensureLoggedIn());
//Github hook endpoints
app.get('/auth/github', passport.authorize('github'), function(req, res){});
app.get('/auth/github/callback', passport.authorize('github', {failureRedirect: '/fail' }), user.addHook, site.hookAdded);

//Caller id hook endpoints
app.get('/auth/rtcgateway', passport.authorize('webrtcgw-authz', {scope: '*',  failureRedirect: '/login'}), user.show);
app.get('/auth/rtcgateway/callback', passport.authorize('webrtcgw-authz', {scope: '*', failureRedirect: '/login'}), 
        user.addHook, site.hookAdded);

app.all('/*', function(req, res,next){
  //Create jsonResponse object on all requests for attaching data to 
  req.jsonResponse = {};
  next();
});
//All browser website routes
app.all('/site*', login.ensureLoggedIn());
app.get('/site/sensors', user.getallgateways, user.getallsensors, site.dashboard);
app.get('/site/basic_rule', user.getallrules, user.getallsensors, user.getallhooks, basicRule.index);
app.post('/site/basic_rule', basicRule.create, site.jsonResponse);
app.get('/site/rule', user.getallrules, user.getallsensors, user.getallhooks,  site.rules);
app.post('/site/rule', rule.create,  site.jsonResponse);
app.get('/site/rule/:ruleId', user.getallrules, user.getallsensors, user.getallhooks,  site.rules);
app.put('/site/rule/:ruleId', rule.update,  site.jsonResponse);
app.delete('/site/rule/:ruleId', rule.delete,  site.jsonResponse);
app.get('/site/sensor/:sensorId', sensor.show);
app.delete('/site/sensor/:sensorId', sensor.remove, site.jsonResponse);
app.post('/site/sensor/:sensorId', sensor.update, site.jsonResponse);
app.post('/site/gateway/:gatewayId', gateway.updateName, site.jsonResponse);

app.get('/site/location', user.getalllocations, user.getallsensors,  site.locations);
app.post('/site/location', mpMiddleware, location.create,  site.jsonResponse);
app.get('/site/location/:locationId', user.getalllocations, user.getallsensors,  site.locations);
app.get('/site/location/:locationId/image', site.getImage);
app.put('/site/location/:locationId', location.update,  site.jsonResponse);
app.delete('/site/location/:locationId', location.delete,  site.jsonResponse);


app.get('/site/action/:sensorId', sensor.showactions);
app.post('/site/action/:sensorId/:action', sensor.doAction, site.jsonResponse);
app.get('/site/connection_key', gateway.socketkey);

app.param('sensorId', function(req, res, next, id){
  console.log('looking for ', id); 
  if(!req.user){
    console.log("No user associated with request... forbidden");
    return res.send(403);
  }

  Sensor.findOne({_id: id, user: req.user._id})
    .exec(function(err, sensor){
      if(err) return next(err);
      if(!sensor){
        next();
      }else{
        req.sensor = sensor;
        next();
      }
    });
});

app.param('gatewayId', function(req, res, next, id){
  console.log('looking for ', id); 
  if(!req.user){
    console.log("No user associated with request... forbidden");
    return res.send(403);
  }

  Gateway.findOne({_id: id, user: req.user._id})
    .exec(function(err, gw){
      if(err) return next(err);
      if(!gw){
        return res.send(404);
      }else{
        req.gateway = gw;
        next();
      }
    });
});


app.param('ruleId', function(req, res, next, id){
  console.log('looking for ', id); 
  if(!req.user){
    console.log("No user associated with request... forbidden");
    return res.send(403);
  }

  Rule.findOne({_id: id, user: req.user._id})
    .exec(function(err, rule){
      if(err) return next(err);
      if(!rule){
        return res.send(404);
      }else{
        req.rule = rule;
        next();
      }
    });
});

app.param('locationId', function(req, res, next, id){
  console.log('looking for ', id); 
  if(!req.user){
    console.log("No user associated with request... forbidden");
    return res.send(403);
  }

  Location.findOne({_id: id, user: req.user._id})
    .exec(function(err, location){
      if(err) return next(err);
      if(!rule){
        return res.send(404);
      }else{
        req.location = location;
        next();
      }
    });
});




app.all('/api*', passport.authenticate('token', { session: false }));
app.get('/api/userinfo', user.info);
app.get('/api/socketkey', user.socketkey);
app.post('/api/connection_key', gateway.findOrCreate, gateway.socketkey);

app.get('/api/sensor', user.getallsensors, site.jsonResponse);

app.get('/api/sensor/:sensorMac', sensor.get, site.jsonResponse);
app.post('/api/sensor/:sensorMac', sensor.read);
app.put('/api/sensor/:sensorMac', sensor.read);
app.post('/secret/:userId/sensor/:sensorMac', sensor.read);
app.put('/secret/:userId/sensor/:sensorMac', sensor.read);

app.get('/api/action/:sensorMac', sensor.getActions, site.jsonResponse);
app.post('/api/action/:sensorMac/:action', sensor.doAction, site.jsonResponse);

app.all('/admin*', login.ensureLoggedIn(), function(req, res, next){
      console.log('checking if user is admin');
      if(req.user.isAdmin)
        next();
      else
        res.render('admin/notAdmin');
});

event_service.on('gateway', gateway.update);


app.get('/admin', admin.show);
//app.post('/admin/email', admin.email);
app.get('/admin/all', admin.all);
app.get('/admin/user/:userId', admin.showuser);
app.post('/admin/user/:userId', admin.edituser);
app.param('userId', function(req, res, next, id){
  User.findOne({_id: id})
  .exec(function(err, user){
    if (err) return next(err);
    if (!user) return next(new Error('Failed to load User ' + id));
    req.user = user;
    req.profile = user;
    next();
  });
});

app.param('sensorMac', function(req, res, next, id){
  if(!req.user){
    console.log("No user associated with request... forbidden");
    res.send(403);
  }
  Sensor.findOne({mac:id, user: req.user._id})
    .exec(function(err, sensor){
      if (err) return next(err);
      if (!sensor){
        req.sensor = false;
        next();
      }else{
        req.sensor = sensor;
        next();
      }
    });
  });

}
