(function($) {
  
  var first = true;
  /**
   * Custom class for Rules popovers. Extends Bootstrap Popover
   */
  function RulesPopover(element, options) {
    this.init('popover', element, $.extend({'trigger': 'manual'}, options));
    return this;
  }
  $.extend(RulesPopover.prototype, $.fn.popover.Constructor.prototype, {
    
    hasContent: function() {
      return (this.$tip || ($(this.$element.attr('href')).length != 0));
    }
    
    , setContent: function() {
      this.tip().removeClass('fade top bottom left right in');
    }
    
    , tip: function() {
      if (!this.$tip)
        this.$tip = $(this.$element.attr('href'));
      
      return this.$tip;
    }
    
    , reposition: function() {
      var $tip = this.tip();
      var tp;
      var placement = typeof this.options.placement == 'function' ?
          this.options.placement.call(this, $tip[0], this.$element[0]) :
          this.options.placement;
      var pos = this.getPosition();
      
      var actualWidth = $tip[0].offsetWidth;
      var actualHeight = $tip[0].offsetHeight;

      switch (placement) {
        case 'bottom':
          tp = {top: pos.top + pos.height, left: pos.left + pos.width / 2 - actualWidth / 2};
          break
        case 'top':
          tp = {top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2};
          break
        case 'left':
          tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth};
          break
        case 'right':
          tp = {top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width};
          break
      }

      this.applyPlacement(tp, placement);
    }
    
  });

  /**
   * Rules modal dialog
   */
  function RulesDialog(target, options) {
    this._options = $.extend({}, this._options, options);
    this._target = $(target);
    
    this._initDialog();

    return this;
  }
  $.extend(RulesDialog.prototype, {
    
    _options: {
      nextButton: '.btn-next'
      , prevButton: '.btn-prev'
      , stepLinks: '.link-step'
      , popupTriggers: '[data-toggle=rules-popover]'
    }
    
    /**
     * Initialize the dialog
     */
    , _initDialog: function() {
      var self = this;
      
      this._nextButton = this._target.find(this._options.nextButton);
      this._prevButton = this._target.find(this._options.prevButton)
      
      this._nextButton.click(function() { return self._nextClicked(this); });
      this._prevButton.click(function() { return self.movePrevious(this); });
      this._target.find(this._options.stepLinks).click(function() {
        var link = $(this);
        
        if (link.data('event-type'))
          self._target.data('event-type', link.data('event-type'));

        if (link.data('sub-type'))
          self._target.data('sub-type', link.data('sub-type'));

        return self.moveToStep(link.attr('href'), this);
      });
      
      this._target.find('form').submit(function(e) { return self._submit(e, $(this)); });
      
      $(this._options.popupTriggers).each(function() {
        var $elem = $(this);
        var $step = $elem.closest('.step');
        var pop = new RulesPopover(this);
        $elem.data('popover', pop);

        $elem.click(function() {
          if ($step.data('currentPopover') == pop) {
            pop.hide();
            $step.removeData('currentPopover');
            return false;
          }
        
          if ($step.data('currentPopover'))
            $step.data('currentPopover').hide();
        
          pop.show();
          $step.data('currentPopover', pop);
        
          return false;
        });

      });
      
      this._target.find('.btn-sensor').click(function() {
        var $btn = $(this);
        var $step = $btn.closest('.step');
        
        if ($btn.hasClass('active')) {
          $btn.removeClass('active');
        } else {
          $step.find('.btn-sensor.active').removeClass('active');
          $btn.addClass('active');
          
          if ( ($btn.data('toggle') != 'rules-popover') && ($step.data('currentPopover')) ) {
            $step.data('currentPopover').hide();
            $step.removeData('currentPopover');
          }
        }
        
        $step.trigger('selectionchange');
      });
      
      this._target.find('.step').on('selectionchange', function() { self._setStepState($(this)); });
      
      this._target.find('.popover .required:input').on('change', function() {
        self._setStepState($(this).closest('.step'));
      });
      
      this._target.find('.step').on('selectioncomplete', function() { self._setDataForStep($(this)); });
      
      this._target.find('.link-step').click(function() {
        var link = $(this);

        if (link.data('rule-type'))
          self._target.find('#rule_type').val(link.data('rule-type'));

        if (link.data('hook-id'))
          self._target.find('#rule_hookId').val(link.data('hook-id'));

        return false;
      });
      
      this._setButtonState();
    }
    
    /**
     * Handle a click on the next button
     */
    , _nextClicked: function() {
      var current = this._target.find('.step.current');

      if (current.hasClass('final') && this._isNextReady(current)) {
        this._saveRule();
        return false;
      } else {
        return this.moveNext();
      }
    }
    
    /**
     * Move to the next step
     */
    , moveNext: function() {
      var current = this._target.find('.step.current');
      var next = current.data('next-step');
      
      if ((!next) || (next === '')) return false;
      
      return this.moveToStep(next);
    }
    
    /**
     * Move to the previous step
     */
    , movePrevious: function() {
      var current = this._target.find('.step.current');
      var prev = current.prevAll('.step.past');
      
      if (prev.length == 0) return false;
      
      return this.moveToStep(prev.first());
    }
    
    /**
     * Move to a specific step
     */
    , moveToStep: function(step) {
      var newCurrent = $(step);
      var newPast = newCurrent.prevAll('.step.current');
      var newFuture = newCurrent.nextAll('.step.current, .step.past');
      
      newCurrent.removeClass('future').removeClass('past').addClass('current');
      newPast.removeClass('current').removeClass('future').addClass('past');
      newFuture.removeClass('current').removeClass('past').addClass('future');
      
      this._stepChanged();
      
      return false;
    }
    
    /**
     * Invoked internally to update control state when the current step has changed
     */
    , _stepChanged: function() {
      // identify buttons with two lines of text
      this._target.find('.step.current .btn-sensor').not('.lines-2').find('.text').each(function() {
        var lines = this.getClientRects();
        if (lines.length > 1)
          $(this).closest('.btn-sensor').addClass('lines-2');
      });
      
      var eventType = this._target.data('event-type');
      var subType = String(this._target.data('sub-type'));
      this._target.find('.step.current .btn-sensor').each(function() {
        var btn = $(this);
        var reqs = btn.data('display-requirements');
        if (!reqs) return;
        
        var parts = reqs.split(/\|/);
        var events = parts[0].split(',');
        var subs = parts[1].split(',');
        if (subs.length > 0 && subs[0] == '') subs.shift();
        
        var display = true;
        if (events.indexOf('any') == -1 && events.indexOf(eventType) == -1)
          display = false;
        
        if (display && subs.length > 0 && subs.indexOf(subType) == -1)
          display = false;
        
        if (display)
          btn.show();
        else {
          btn.removeClass('active');
          btn.hide();
        }
      });
      
      
      var current = this._target.find('.step.current');
      if (current.hasClass('final')) {
        this._nextButton.find('.txt').text('Save');
        this._nextButton.find('.icon').removeClass('icon-next').addClass('icon-save');
        this._nameRule();
      } else {
        this._nextButton.find('.txt').text('Next');
        this._nextButton.find('.icon').removeClass('icon-save').addClass('icon-next');
      }
      

      if (current.attr('id') == 'step-select-action') {
        current.find('.btn-sensor:visible').each(function(idx) {
          var placement = (idx % 4 == 0) ? 'right' : 'bottom';
          var pop = $(this).data('popover');
          if (pop) pop.options.placement = placement;
        });
        
        var currentPop = current.data('currentPopover');
        if (currentPop) {
          if (current.find('.btn-sensor.active').length > 0) {
            currentPop.reposition();
          } else {
            currentPop.hide();
            current.removeData('currentPopover');
          }
        }
      }


      this._setButtonState();
    }
    
    /**
     * Set the navigation button state
     */
    , _setButtonState: function() {
      var current = this._target.find('.step.current');
      var hasPrevious = current.prevAll('.step.past').length > 0;
      
      if (hasPrevious)
        this._prevButton.prop('disabled', false)
      else
        this._prevButton.prop('disabled', true)

      this._setNextButtonState(current);
    }
    
    /**
     * Set the navigation state of only next button given the current step
     */
    , _setNextButtonState: function(current) {
      var hasNext = current.data('next-step');

      if ((current.hasClass('final') || hasNext) && this._isNextReady(current))
        this._nextButton.prop('disabled', false)
      else
        this._nextButton.prop('disabled', true)
    }
    
    
    /**
     * Check the ready state of the next button based on validation rules
     */
    , _isNextReady: function(step) {
      if (!step.hasClass('selection-required')) return true;
      
      var selected = step.find('.btn-sensor.active');
      if (selected.length !== 1) return false;
      if (selected.data('toggle') != 'rules-popover') return true;
      
      var popover = $(selected.attr('href'));
      var missing = false;
      popover.find('.required:input').each(function() {
        var val = $(this).val();
        if ((!val) || (val === '')) {
          missing = true;
          return false;
        }
      });
      
      return (!missing);
    }
    
    /**
     * Perform step state change
     */
    , _setStepState: function(step) {
      if (step.hasClass('current'))
        this._setNextButtonState(step);
    
      if (!this._nextButton.prop('disabled'))
        step.trigger('selectioncomplete');
    }
    
    /**
     * Set data for selection in step
     */
    , _setDataForStep: function(step) {
      var self = this;
      
      step.find('.btn-sensor.active').each(function() {
        var btn = $(this);
        
        if (btn.data('event-type'))
          self._target.data('event-type', btn.data('event-type'));

        if (btn.data('sub-type'))
          self._target.data('sub-type', btn.data('sub-type'));

        if (btn.data('sensor-type'))
          self._target.find('#rule_sensorType').val(btn.data('sensor-type'));

        if (btn.data('action-type'))
          self._target.find('#rule_actionType').val(btn.data('action-type'));
      });
    }
    
    /**
     * Give the rule a default name
     */
    , _nameRule: function() {
      var input = this._target.find('.rule-name');
      if (input.val() == '')
        input.val('My new rule');
    }
    
    /**
     * Save the rule
     */
    , _saveRule: function() {
      var form = this._target.find('form').submit();
    }
    
    /**
     * Handle form submission
     */
    , _submit: function(e, form) {
      e.preventDefault();

      var current = this._target.find('.step.current');
      if ( (!current.hasClass('final')) || (!this._isNextReady(current)) )
        return false;
        
      this._nextButton.prop('disabled', true);
      
      var self = this;
      
      $.ajax({
        type: 'POST',
        url: form.attr('action'), 
        data: form.serialize(),
        dataType: 'json',
        success: function(data) {
          self._target.modal('hide');
          if (data.success)
            window.location = '/site/basic_rule';
          else {
            $('#error-container').append(
              '<div class="alert alert-error">'+
                '<button type="button" class="close" data-dismiss="alert">&times;</button>'+
                '<strong>Error</strong> '+data.message+
              '</div>');
            self._nextButton.prop('disabled', false);
          }
        }
      });
      
      return false;
    }
    
  });
  
  
  /**
   * Page setup
   */
  $(function() {
    var newRulesDialog = new RulesDialog('#rule_new');
  });
  
})(jQuery);
